@@ -69,6 +69,12 @@ sub exp_mangoldt {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::exp_mangoldt($n);
 }
+sub hclassno {
+  my($n) = @_;
+  return 0 if defined $n && int($n) < 0;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::hclassno($n);
+}
 
 
 sub next_prime {
@@ -132,6 +138,10 @@ sub print_primes {
   my($low,$high,$fd) = @_;
   return Math::Prime::Util::PP::print_primes($low,$high,$fd);
 }
+sub sieve_range {
+  my($n,$width,$depth) = @_;
+  return Math::Prime::Util::PP::sieve_range($n,$width,$depth);
+}
 sub sieve_prime_cluster {
   my($low,$high,@cl) = @_;
   return Math::Prime::Util::PP::sieve_prime_cluster($low,$high,@cl);
@@ -164,11 +174,18 @@ sub nth_ramanujan_prime {
 *is_bpsw_prime     = \&Math::Prime::Util::PP::is_bpsw_prime;
 
 sub is_pseudoprime {
-  my($n, $base) = @_;
+  my($n, @bases) = @_;
   return 0 if defined $n && int($n) < 0;
   _validate_positive_integer($n);
-  _validate_positive_integer($base) if defined $base;
-  return Math::Prime::Util::PP::is_pseudoprime($n, $base);
+  croak "No bases given to is_strong_pseudoprime" unless @bases;
+  return Math::Prime::Util::PP::is_pseudoprime($n, @bases);
+}
+sub is_euler_pseudoprime {
+  my($n, @bases) = @_;
+  return 0 if defined $n && int($n) < 0;
+  _validate_positive_integer($n);
+  croak "No bases given to is_euler_pseudoprime" unless @bases;
+  return Math::Prime::Util::PP::is_euler_pseudoprime($n, @bases);
 }
 sub is_strong_pseudoprime {
   my($n, @bases) = @_;
@@ -261,6 +278,18 @@ sub is_carmichael {
   #_validate_positive_integer($n);
   return Math::Prime::Util::PP::is_carmichael($n);
 }
+sub is_quasi_carmichael {
+  my($n) = @_;
+  #_validate_positive_integer($n);
+  return Math::Prime::Util::PP::is_quasi_carmichael($n);
+}
+sub is_primitive_root {
+  my($a,$n) = @_;
+  $n = -$n if defined $n && $n < 0;
+  _validate_positive_integer($a);
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::is_primitive_root($a,$n);
+}
 
 
 sub lucas_sequence {
@@ -476,6 +505,32 @@ sub invmod {
   _validate_integer($n);
   return Math::Prime::Util::PP::invmod($a,$n);
 }
+sub sqrtmod {
+  my ($a, $n) = @_;
+  _validate_integer($a);
+  _validate_integer($n);
+  return Math::Prime::Util::PP::sqrtmod($a,$n);
+}
+sub addmod {
+  my ($a, $b, $n) = @_;
+  _validate_integer($a); _validate_integer($b>=0?$b:-$b); _validate_integer($n);
+  return Math::Prime::Util::PP::addmod($a,$b, $n);
+}
+sub mulmod {
+  my ($a, $b, $n) = @_;
+  _validate_integer($a); _validate_integer($b>=0?$b:-$b); _validate_integer($n);
+  return Math::Prime::Util::PP::mulmod($a,$b, $n);
+}
+sub divmod {
+  my ($a, $b, $n) = @_;
+  _validate_integer($a); _validate_integer($b>=0?$b:-$b); _validate_integer($n);
+  return Math::Prime::Util::PP::divmod($a,$b, $n);
+}
+sub powmod {
+  my ($a, $b, $n) = @_;
+  _validate_integer($a); _validate_integer($b>=0?$b:-$b); _validate_integer($n);
+  return Math::Prime::Util::PP::powmod($a,$b, $n);
+}
 sub sqrtint {
   my($n) = @_;
   _validate_integer($n);
