@@ -4,7 +4,7 @@ use warnings;
 
 BEGIN {
   $ntheory::AUTHORITY = 'cpan:DANAJ';
-  $ntheory::VERSION = '0.56';
+  $ntheory::VERSION = '0.58';
 }
 
 BEGIN {
@@ -49,8 +49,9 @@ See L<Math::Prime::Util> for complete documentation.
 
 =head2 PROBABLE PRIME TESTS
 
-  is_pseudoprime(n,bases)                  Fermat probable prime tests
-  is_strong_pseudoprime(n,bases)           Miller-Rabin tests to bases
+  is_pseudoprime(n,bases)                  Fermat probable prime test
+  is_euler_pseudoprime(n,bases)            Euler test to bases
+  is_strong_pseudoprime(n,bases)           Miller-Rabin test to bases
   is_lucas_pseudoprime(n)                  Lucas test
   is_strong_lucas_pseudoprime(n)           strong Lucas test
   is_almost_extra_strong_lucas_pseudoprime(n, [incr])   AES Lucas test
@@ -67,8 +68,9 @@ See L<Math::Prime::Util> for complete documentation.
 
   primes([start,] end)                array ref of primes
   twin_primes([start,] end)           array ref of twin primes
-  sieve_prime_cluster(start, end, @C) list of prime k-tuples
   ramanujan_primes([start,] end)      array ref of Ramanujan primes
+  sieve_prime_cluster(start, end, @C) list of prime k-tuples
+  sieve_range(n, width, depth)        sieve out small factors to depth
   next_prime(n)                       next prime > n
   prev_prime(n)                       previous prime < n
   prime_count(n)                      count of primes <= n
@@ -85,6 +87,8 @@ See L<Math::Prime::Util> for complete documentation.
   twin_prime_count_approx(n)          fast approx count of twin primes
   nth_twin_prime(n)                   the nth twin prime (n=1 returns 3)
   nth_twin_prime_approx(n)            fast approximate nth twin prime
+  ramanujan_prime_count(n)            count of Ramanujan primes <= n
+  ramanujan_prime_count(start, end)   count of Ramanujan primes in range
   nth_ramanujan_prime(n)              the nth Ramanujan prime (Rn)
   legendre_phi(n,a)                   # below n not div by first a primes
   prime_precalc(n)                    precalculate primes to n
@@ -111,6 +115,7 @@ See L<Math::Prime::Util> for complete documentation.
   forcomp { ... } n [,{...}]          loop over integer compositions
   forcomb { ... } n, k                loop over combinations
   forperm { ... } n                   loop over permutations
+  formultiperm { ... } \@n            loop over multiset permutations
   prime_iterator                      returns a simple prime iterator
   prime_iterator_object               returns a prime iterator object
 
@@ -127,20 +132,34 @@ See L<Math::Prime::Util> for complete documentation.
   random_shawe_taylor_prime(n)        random n-bit prime with S-T alg.
   random_shawe_taylor_prime_with_cert(n) as above including certificate
 
-=head2 MATH
+=head2 LISTS
 
   vecsum(@list)                       integer sum of list
   vecprod(@list)                      integer product of list
   vecmin(@list)                       minimum of list of integers
   vecmax(@list)                       maximum of list of integers
-  vecreduce { ... } @list             reduce / left fold applied to list
   vecextract(\@list, mask)            select from list based on mask
+  vecreduce { ... } @list             reduce / left fold applied to list
+  vecall { ... } @list                return true if all are true
+  vecany { ... } @list                return true if any are true
+  vecnone { ... } @list               return true if none are true
+  vecnotall { ... } @list             return true if not all are true
+  vecfirst { ... } @list              return first value that evals true
+
+=head2 MATH
+
+  todigits(n[,base[,len]])            convert n to digit array in base
+  todigitstring(n[,base[,len]])       convert n to string in base
+  fromdigits(\@d,[,base])             convert base digit vector to number
+  fromdigits(str,[,base])             convert base digit string to number
   sumdigits(n)                        sum of digits, with optional base
   is_power(n)                         return k if n = p^k for integer p, max k
   is_power(n,k)                       return 1 if n = p^k for integer p and k
   is_power(n,k,\$root)                as above but set root to p.
   is_square_free(n)                   return true if no repeated factors
   is_carmichael(n)                    is n a Carmichael number
+  is_quasi_carmichael(n)              is n a quasi-Carmichael number
+  is_primitive_root(r,n)              is r a primitive root mod n
   sqrtint(n)                          integer square root
   gcd(@list)                          greatest common divisor
   lcm(@list)                          least common multiple
@@ -154,7 +173,12 @@ See L<Math::Prime::Util> for complete documentation.
   valuation(n,k)                      number of times n is divisible by k
   hammingweight(n)                    population count (# of binary 1s)
   kronecker(a,b)                      Kronecker (Jacobi) symbol
+  addmod(a,b,n)                       a + b mod n
+  mulmod(a,b,n)                       a * b mod n
+  divmod(a,b,n)                       a / b mod n
+  powmod(a,b,n)                       a ^ b mod n
   invmod(a,n)                         inverse of a modulo n
+  sqrtmod(a,n)                        modular square root
   moebius(n)                          Moebius function of n
   moebius(beg, end)                   array of Moebius in range
   mertens(n)                          sum of Moebius for 1 to n
@@ -168,6 +192,7 @@ See L<Math::Prime::Util> for complete documentation.
   znprimroot(n)                       smallest primitive root
   chebyshev_theta(n)                  first Chebyshev function
   chebyshev_psi(n)                    second Chebyshev function
+  hclassno(n)                         Hurwitz class number H(n) * 12
   ramanujan_tau(n)                    Ramanujan's Tau function
   consecutive_integer_lcm(n)          lcm(1 .. n)
   lucasu(P, Q, k)                     U_k for Lucas(P,Q)
@@ -197,7 +222,7 @@ See L<Math::Prime::Util> for complete documentation.
 
 =head1 COPYRIGHT
 
-Copyright 2011-2015 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
+Copyright 2011-2016 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
