@@ -66,6 +66,8 @@ extern UV binomial(UV n, UV k);
 extern IV gcdext(IV a, IV b, IV* u, IV* v, IV* s, IV* t); /* Ext Euclidean */
 extern UV modinverse(UV a, UV p);              /* Returns 1/a mod p */
 extern UV divmod(UV a, UV b, UV n);            /* Returns a/b mod n */
+extern int sqrtmod(UV* s, UV a, UV p);         /* sqrt(a) mod p */
+extern int sqrtmod_composite(UV* s, UV a,UV n);/* sqrt(a) mod n */
 extern UV chinese(UV* a, UV* n, UV num, int *status); /* Chinese Remainder */
 
 extern UV totient(UV n);
@@ -75,19 +77,22 @@ extern UV carmichael_lambda(UV n);
 extern UV jordan_totient(UV k, UV n);
 extern UV znprimroot(UV n);
 extern UV znorder(UV a, UV n);
-extern int is_primitive_root(UV a, UV p);
+extern int is_primitive_root(UV a, UV n, int nprime);
 #define is_square_free(n)  (moebius(n) != 0)
 extern int is_carmichael(UV n);
-extern int is_quasi_carmichael(UV n);
+extern UV  is_quasi_carmichael(UV n);  /* Returns number of bases */
 
 extern IV stirling2(UV n, UV m);
 extern IV stirling1(UV n, UV m);
 
+extern IV hclassno(UV n);
 extern IV ramanujan_tau(UV n);
 
 extern int strnum_minmax(int min, char* a, STRLEN alen, char* b, STRLEN blen);
 
 extern int from_digit_string(UV* n, const char* s, int base);
+extern int from_digit_to_UV(UV* rn, UV* r, int len, int base);
+extern int from_digit_to_str(char** rstr, UV* r, int len, int base);
 extern int to_digit_array(int* bits, UV n, int base, int length);
 extern int to_digit_string(char *s, UV n, int base, int length);
 
